
ANONYMOUS_MAILERS = [
    '3a.by',
    'dispostable.com',
    'grandmamail.com',
    'harakirimail.com',
    'lackmail.ru',
    'maildrop.cc',
    'mailinator.com',
]
"""Known anonymous mailing hosts to be used in BlacklistedDomain."""
